//======================================================================
//  This file is part of the M.A.RL.I.E. software library.
//
//  Copyright (C) 2019 by CRS4 Visual Computing Group, Italy
//
//  For more information, visit the CRS4 Visual Computing Group 
//  web pages at http://vic.crs4.it/download
//  This file may be used under the terms of the GNU General Public
//  License as published by the Free Software Foundation and appearing
//  in the file LICENSE included in the packaging of this file.
//
//  CRS4 reserves all rights not expressly granted herein.
//  
//  This file is provided AS IS with NO WARRANTY OF ANY KIND, 
//  INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS 
//  FOR A PARTICULAR PURPOSE.
//======================================================================

class ShaderSetup {
	
	constructor(renderer) {
		this.renderer = renderer;

		this.commonParams = {
			useDirLight : true,
			lightDir : [0,0,1],
			lightSpot : [0,0,200],
		}

		this.baseParams = {
			firstTexNum: 0,
			hasAnnotations: false,
			drawAnnotations: false,
			renderMode: 0,
			normalEnhancementK: 0,
			normalEnhancementLOD: 1,
			brightness: 1,
			gamma: 2
		}

		this.lensParams = {
			lensPos: [0,0],
			lensRadius: 50.0,
			lensAlpha: 1.0,
			firstTexNum: 5,
			hasAnnotations: false,
			drawAnnotations: false,
			renderMode: 0,
			normalEnhancementK: 0,
			normalEnhancementLOD: 1,
			brightness: 1,
			gamma: 2
		}

	}

	updateCommon() {
		let k = [0,0,0,0];
		if(this.commonParams.useDirLight)
			k = [ this.commonParams.lightDir[0], this.commonParams.lightDir[1], this.commonParams.lightDir[2], 0];
		else
			k = [ this.commonParams.lightSpot[0], this.commonParams.lightSpot[1], this.commonParams.lightSpot[2], 1];
	
		this.renderer.gl.uniform4fv(this.renderer.getAttribID("uLightInfo"), k);
	}

	updateForBase() {
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uLensPass"), false);

		this.updateCommon();

		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexNormals"), this.baseParams.firstTexNum);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexKD"), this.baseParams.firstTexNum+1);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexKS"), this.baseParams.firstTexNum+2);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexGloss"), this.baseParams.firstTexNum+3);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexAnnot"), this.baseParams.hasAnnotations ? this.baseParams.firstTexNum+4 : this.baseParams.firstTexNum);

		this.renderer.gl.uniform1i(this.renderer.getAttribID("uDrawAnnotations"), this.baseParams.drawAnnotations);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uRenderMode"), this.baseParams.renderMode);

		this.renderer.gl.uniform2f(this.renderer.getAttribID("uNormalEnhancementParams"), this.baseParams.normalEnhancementK, this.baseParams.normalEnhancementLOD);
		this.renderer.gl.uniform2f(this.renderer.getAttribID("uRenderCorrections"), this.baseParams.brightness, this.baseParams.gamma);	
	}

	updateForLens() {
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uLensPass"), true);

		this.updateCommon();

		this.renderer.gl.uniform4f(this.renderer.getAttribID("uLensInfo"), this.lensParams.lensPos[0], this.lensParams.lensPos[1], this.lensParams.lensRadius, this.lensParams.lensAlpha);
		
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexNormals"), this.lensParams.firstTexNum);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexKD"), this.lensParams.firstTexNum+1);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexKS"), this.lensParams.firstTexNum+2);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexGloss"), this.lensParams.firstTexNum+3);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uTexAnnot"), this.lensParams.hasAnnotations ? this.lensParams.firstTexNum+4 : this.lensParams.firstTexNum);

		this.renderer.gl.uniform1i(this.renderer.getAttribID("uDrawAnnotations"), this.lensParams.drawAnnotations);
		this.renderer.gl.uniform1i(this.renderer.getAttribID("uRenderMode"), this.lensParams.renderMode);

		this.renderer.gl.uniform2f(this.renderer.getAttribID("uNormalEnhancementParams"), this.lensParams.normalEnhancementK, this.lensParams.normalEnhancementLOD);
		this.renderer.gl.uniform2f(this.renderer.getAttribID("uRenderCorrections"), this.lensParams.brightness, this.lensParams.gamma);	
	}

}
